Attribute VB_Name = "modRunMenu"
Option Explicit

' Declare our API calls that we are going to use.
Public Declare Function GetCursorPos Lib "user32" (lpPoint As POINTAPI) As Long
Public Declare Function GetMenu Lib "user32" (ByVal hwnd As Long) As Long
Public Declare Function GetMenuItemCount Lib "user32" (ByVal hMenu As Long) As Long
Public Declare Function GetSubMenu Lib "user32" (ByVal hMenu As Long, ByVal nPos As Long) As Long
Public Declare Function GetMenuItemID Lib "user32" (ByVal hMenu As Long, ByVal nPos As Long) As Long
Public Declare Function GetMenuString Lib "user32" Alias "GetMenuStringA" (ByVal hMenu As Long, ByVal wIDItem As Long, ByVal lpString As String, ByVal nMaxCount As Long, ByVal wFlag As Long) As Long
Public Declare Function SendMessageLong Lib "user32" Alias "SendMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Public Declare Function WindowFromPoint Lib "user32" (ByVal xPoint As Long, ByVal yPoint As Long) As Long

' Declare our constant(s).
Public Const MAX_PATH = 260
Public Const WM_COMMAND = &H111

' Declare our type(s).
Public Type POINTAPI
        x As Long
        y As Long
End Type

Public Sub RunMenu(ByVal hwnd As Long, ByVal strMenuString As String, Optional ByVal blnFilterAmps As Boolean = False)
    ' This is a pretty long sub but I'll try to
    ' explain it as best as I can. This sub is
    ' used to call a menu item on another window.
    
    ' Input Arguments:
    '   hwnd:          Handle of the window that
    '                  has the menu that we are
    '                  going to call.
    '   strMenuString: The text of the menu item.
    '   blnFilterAmps: Determines whether or not
    '                  to filter the Ampersands (&)
    '                  from both the inputted menu
    '                  string AND the original menu
    '                  string TO COMPARE.
    
    ' If we run into any errors, just continue
    ' with the sub.
    On Error Resume Next
    
    ' Declare our BIG load of variables.
    Dim lngMenuHandle As Long
    Dim lngMenuCount As Long
    Dim lngMenuCurrent As Long
    Dim lngSubMenuHandle As Long
    Dim lngSubMenuItemCount As Long
    Dim lngSubMenuCurrent As Long
    Dim lngSubMenuItemID As Long
    Dim lngSubMenuItemIDTemp As Long
    Dim strSubMenuItemText As String
    Dim lngSubMenuHandle2 As Long
    Dim lngSubMenuItemCount2 As Long
    Dim lngSubMenuCurrent2 As Long
    Dim lngSubMenuItemID2 As Long
    Dim lngSubMenuItemIDTemp2 As Long
    Dim strSubMenuItemText2 As String
    Dim i
    
    ' Get the MAIN menu handle of the window.
    lngMenuHandle = GetMenu(hwnd)
    ' Get the number of VISIBLE MENUS on the
    ' window.
    lngMenuCount = GetMenuItemCount(lngMenuHandle)
    
    ' Create a loop to go through the menu items.
    For lngMenuCurrent = 0 To lngMenuCount - 1
        ' Get the handle of the current menu.
        lngSubMenuHandle = GetSubMenu(lngMenuHandle, lngMenuCurrent)
        ' Get the number of visible items on the
        ' current menu.
        lngSubMenuItemCount = GetMenuItemCount(lngSubMenuHandle)
        ' Create another loop to go through the
        ' menu items of the current menu.
        For lngSubMenuCurrent = 0 To lngSubMenuItemCount - 1
            ' Get the handle of the current menu
            ' item.
            lngSubMenuHandle2 = GetSubMenu(lngSubMenuHandle, lngSubMenuCurrent)
            ' Get the item count of the current
            ' menu item. If this is equal to -1
            ' then it means that the current item
            ' IS NOT a submenu!
            lngSubMenuItemCount2 = GetMenuItemCount(lngSubMenuHandle2)
            
            ' If it is equal to -1, it isn't a
            ' submenu.
            If lngSubMenuItemCount2 = -1 Then
                ' Get the menu item ID of the
                ' current item.
                lngSubMenuItemID = GetMenuItemID(lngSubMenuHandle, lngSubMenuCurrent)
                ' Fill the text variable with null
                ' characters.
                strSubMenuItemText = String(MAX_PATH, Chr(0))
                
                ' Fill the text variable with the
                ' menu item string.
                Call GetMenuString(lngSubMenuHandle, lngSubMenuItemID, strSubMenuItemText, MAX_PATH, 1)
                
                ' Chop off the null characters
                ' that are left over from the
                ' filling.
                strSubMenuItemText = Left(strSubMenuItemText, InStr(1, strSubMenuItemText, Chr(0)) - 1)
                ' If the user selected to filter out the Ampersands (&)
                If blnFilterAmps Then
                    ' Filter out the Ampersands (&).
                    strSubMenuItemText = Replace(strSubMenuItemText, "&", vbNullString)
                    strMenuString = Replace(strMenuString, "&", vbNullString)
                End If
                
                ' If the text that we selected
                ' is found within the menu string,
                ' run the menu.
                If InStr(1, LCase(strSubMenuItemText), LCase(strMenuString)) Then
                    ' Set the ID to a temporary
                    ' variable.
                    lngSubMenuItemIDTemp = lngSubMenuItemID
                    ' Run the menu.
                    Call SendMessageLong(hwnd, WM_COMMAND, lngSubMenuItemIDTemp, 0)
                    ' Exit the sub.
                    Exit Sub
                End If
            ' If the count IS NOT equal to -1, it
            ' is a submenu.
            Else
                ' Create yet another loop to go
                ' through the items of the submenu
                ' (this is the part that most
                ' people leave off when they make
                ' this sub).
                For lngSubMenuCurrent2 = 0 To lngSubMenuItemCount2 - 1
                    ' Get the ID of the current
                    ' submenu item.
                    lngSubMenuItemID = GetMenuItemID(lngSubMenuHandle2, lngSubMenuCurrent2)
                    ' Fill the variable with null
                    ' characters.
                    strSubMenuItemText = String(MAX_PATH, Chr(0))
                    
                    ' Fill the variable with the
                    ' menu string.
                    Call GetMenuString(lngSubMenuHandle, lngSubMenuItemID, strSubMenuItemText, MAX_PATH, 1)
                    
                    ' Chop off the null characters
                    ' left over from the filling.
                    strSubMenuItemText = Left(strSubMenuItemText, InStr(1, strSubMenuItemText, Chr(0)) - 1)
                    ' If the user selected to
                    ' filter the Ampersands (&),
                    ' remove them.
                    If blnFilterAmps Then
                        ' Filter out the Ampersands (&).
                        strSubMenuItemText = Replace(strSubMenuItemText, "&", vbNullString)
                        strMenuString = Replace(strMenuString, "&", vbNullString)
                    End If
                    
                    ' If the string we selected
                    ' is found within the menu
                    ' string, run the menu.
                    If InStr(1, LCase(strSubMenuItemText), LCase(strMenuString)) Then
                        ' Set the ID to a temporary variable.
                        lngSubMenuItemIDTemp = lngSubMenuItemID
                        ' Run the menu.
                        Call SendMessageLong(hwnd, WM_COMMAND, lngSubMenuItemIDTemp, 0)
                        ' Exit the sub.
                        Exit Sub
                    End If
                    ' Allow the processor to process other events.
                    DoEvents
                ' Go to the next menu item.
                Next lngSubMenuCurrent2
            End If
            ' Allow the processor to process other events.
            DoEvents
        ' Go to the next menu item.
        Next lngSubMenuCurrent
        ' Allow the processor to process other events.
        DoEvents
    ' Go to the next menu.
    Next lngMenuCurrent
End Sub

Public Function GetWindowOver() As Long
    ' This function is used to get the window
    ' that we are over.
    
    ' Declare out variable(s).
    Dim POS As POINTAPI
    Dim lngWinHwnd As Long
    
    ' Get the cursor position.
    Call GetCursorPos(POS)
    ' Get the handle of the window from the X and
    ' Y of the mouse positon.
    lngWinHwnd = WindowFromPoint(POS.x, POS.y)
    
    ' Set the value of the function the the
    ' handle.
    GetWindowOver = lngWinHwnd
End Function

Public Function Replace(sIn As String, sFind As String, sReplace As String, Optional lStart As Long = 1, _
    Optional iCount As Long = -1, Optional bCompare As VbCompareMethod = vbBinaryCompare) As String
    ' NOTE: THIS FUNCTION WAS TAKEN DIRECTLY FROM
    ' THE MSDN ONLINE LIBRARY (http://msdn.microsoft.com/)
    Dim lC As Long, iPos As Integer, sOut As String
    sOut = sIn
    'Find first instance of sFind in sIn
    iPos = InStr(lStart, sOut, sFind, bCompare)
    If iPos = 0 Then GoTo EndFn:
    Do
        lC = lC + 1 ' Iterate counter that tracks number of hits On sFind
        'Replace the found instance of sFind wit
        '     h sReplace
        sOut = Left(sOut, iPos - 1) & sReplace & Mid(sOut, iPos + Len(sFind))
        'Check if we should exit if we've found
        '     the required number of sFind's
        If iCount <> -1 And lC >= iCount Then Exit Do
        'Check for the next start character of s
        '     Find in sIn
        iPos = InStr(lStart, sOut, sFind, bCompare)
    Loop While iPos > 0
EndFn:
    Replace = sOut
End Function
